package net.locusworks.common.utils;

import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.Objects;
import java.util.Random;

import static net.locusworks.common.Charsets.UTF_8;

public class RandomString {

  public static final String LOWER = "abcdefghijklmnopqrstuvwxyz";
  public static final String UPPER = LOWER.toUpperCase();
  public static final String DIGITS = "0123456789";
  
  public static final String ALPHA_NUMERIC = LOWER + UPPER + DIGITS;
  
  private Random random;
  
  private char[] symbols;

  private static RandomString instance;

  private RandomString() {
   Random random;
   try {
     random = SecureRandom.getInstance("SHA1PRNG");
   } catch (NoSuchAlgorithmException e) {
     random = new SecureRandom();
   }
   init(random);
  }

  private RandomString(Random random) {
    init(random);
  }

  private void init(Random random) {
    this.random = Objects.requireNonNull(random, "Random generator cannot be null");
    this.symbols = ALPHA_NUMERIC.toCharArray();
  }

  private String nextString(int length) {
    if (length < 1) throw new IllegalArgumentException("String Length has to be greater than 0");
    char[] buffer = new char[length];
    for (int index = 0; index < buffer.length; index++) {
     buffer[index] = symbols[random.nextInt(symbols.length)]; 
    }
    return new String(buffer);
  }
  
  public String getString(Integer length) {
    return this.nextString(length);
  }

  public byte[] getBytes(Integer length) {
    return getString(length).getBytes(UTF_8);
  }

  public static RandomString getInstance() {
    if (instance == null) {
      instance = new RandomString();
    }
    return instance;
  }

  public static RandomString newInstance() {
    instance = new RandomString();
    return instance;
  }

  public static RandomString newInstance(Random random) {
    instance = new RandomString(random);
    return instance;
  }
}
