package net.locusworks.common.utils;

import org.junit.jupiter.api.Test;

import java.util.List;
import java.util.Map;

import static org.junit.jupiter.api.Assertions.*;

class SplitterTest {

  @Test
  void testBasicSplitter() {
    List<String> split = Splitter.onSpace().split("Hello World");
    assertNotNull(split);
    assertEquals(2, split.size());
    assertEquals("Hello", split.get(0));
    assertEquals("World", split.get(1));

    split = Splitter.fixedLengthSplit(5).split("HelloWorld");
    assertNotNull(split);
    assertEquals(2, split.size());
    assertEquals("Hello", split.get(0));
    assertEquals("World", split.get(1));

    split = Splitter.onNewLine().split("Hello\nWorld");
    assertNotNull(split);
    assertEquals(2, split.size());
    assertEquals("Hello", split.get(0));
    assertEquals("World", split.get(1));

    split = Splitter.onNewLine().split("Hello\r\nWorld");
    assertNotNull(split);
    assertEquals(2, split.size());
    assertEquals("Hello", split.get(0));
    assertEquals("World", split.get(1));

    String[] array = Splitter.onSpace().splitToArray("Hello World");
    assertNotNull(array);
    assertEquals(2, array.length);
    assertEquals("Hello", array[0]);
    assertEquals("World", array[1]);
  }

  @Test
  void testOmitEmptyStringWithLimits() {
    List<String> split = Splitter.onSpace().split("Hello  World");
    assertEquals(3, split.size());
    split = Splitter.onSpace().omitEmptyStrings().withLimit(1).split("Hello  World");
    assertNotNull(split);
    assertEquals(1, split.size());
  }

  @Test
  void testExceptions() {
    assertThrows(IllegalArgumentException.class, () -> Splitter.onSpace().split(""));
    assertThrows(IllegalArgumentException.class, () -> Splitter.fixedLengthSplit(0).split("Hello World"));
    assertThrows(NullPointerException.class, () -> Splitter.on(null).split("Hello World"));

  }

  @Test
  void testWithKeyValueSeparator() {
    Map<String, String> map = Splitter.on(";").withKeyValueSeparator("=").split("Hello=World;bubba=hotep");
    assertEquals(2, map.size());
    assertEquals("World", map.get("Hello"));
    assertEquals("hotep", map.get("bubba"));
  }

  @Test
  void testWithMapSplitterErrors() {
    assertThrows(IllegalArgumentException.class, () -> Splitter.on(";").withKeyValueSeparator("").split("Hello=;bubba=hotep"));
    assertThrows(IllegalArgumentException.class, () -> Splitter.on(";").withKeyValueSeparator("=").split("Hello=;bubba=hotep"));
    assertThrows(IllegalArgumentException.class, () -> Splitter.on(";").withKeyValueSeparator("=").split(null));
    Map<String, String> map = Splitter.on(";").withKeyValueSeparator("=").skipInvalidKeyValues().split("Hello=;bubba=hotep");
    assertEquals(1, map.size());
    assertEquals("hotep", map.get("bubba"));
  }


}
