/**
 * 
 */
package net.locusworks.test;

import java.util.List;

import net.locusworks.common.immutables.Triplet;
import net.locusworks.common.objectmapper.ObjectMapperHelper;
import net.locusworks.common.utils.Utils;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import static net.locusworks.common.utils.Constants.JUNIT_TEST_CHECK;
import static net.locusworks.common.utils.Constants.LOG4J_CONFIG_PROPERTY;
import static org.junit.jupiter.api.Assertions.*;

/**
 * Test cases to test ObjectMapperHelper.class
 * @author Isaac Parenteau
 * @since 1.0.0-RELEASE
 *
 */
public class ObjectMapperHelperTest {
  
  private static Triplet<String, Integer, String> test;
  
  /**
   * @throws java.lang.Exception exception
   */
  @BeforeAll
  public static void setUpBeforeClass() throws Exception {
    System.setProperty(LOG4J_CONFIG_PROPERTY, "log4j2-test.xml");
    System.setProperty(JUNIT_TEST_CHECK, "true");
    test = new Triplet<String, Integer, String>("Hello", 24, "World");
  }
  
  @AfterAll
  public static void tearDownAfterClass() throws Exception {
    System.clearProperty(LOG4J_CONFIG_PROPERTY);
    System.clearProperty(JUNIT_TEST_CHECK);
  }

  @Test
  public void testWrite() {
    String value = ObjectMapperHelper.writeValue(test).getResults();
    assertTrue(value != null && !value.trim().isEmpty());
  }
  
  @Test
  public void testRead() {
    String value = ObjectMapperHelper.writeValue(test).getResults();
    assertTrue(value != null && !value.trim().isEmpty());
    
    Triplet<?, ?, ?> tmp = ObjectMapperHelper.readValue(value, Triplet.class).getResults();
    assertNotNull(tmp);
    assertEquals(tmp, test);
  }
  
  @SuppressWarnings("rawtypes")
  @Test
  public void testListWriteRead() {
    List<Triplet<String, Integer, String>> htrList = Utils.toList(test);
    String value = ObjectMapperHelper.writeValue(htrList).getResults();
    assertTrue(value != null && !value.trim().isEmpty());
    List<Triplet> tmpList = ObjectMapperHelper.readListValue(value, Triplet.class).getResults();
    assertTrue(tmpList != null && !tmpList.isEmpty());
  }

}
